/*  Minibar - protos.h "prototype definitions"
    Copyright (C) 2002-2003 Ray Vanderborght (depsypher@users.sf.net)
    
    This file is part of Minibar.

    Minibar is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Minibar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Minibar; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "minibar.h"
#include "PalmOS.h"

//===========================================================================
// Type definitions
//===========================================================================

typedef struct {
	UInt8 nName;
	UInt8 nAmount;		// value in 1/4 increments (allows for 0 to 63 & 3/4)
	UInt8 nUnits;		// index in units list (defined in the .rcp file)
	UInt8 nDescriptors;	// the number of descriptors (currently only 0 or 1)
	UInt8 nDescriptor;	// the index of the descriptor in the appinfo block
} IngredientType;

struct IngredientNodeType {
	IngredientType ingredient;
	struct IngredientNodeType *pNext;
	struct IngredientNodeType *pPrevious;
};
typedef struct IngredientNodeType IngredientNode;
typedef IngredientNode* IngredientNodePtr;

typedef struct {
	char **psArray;
	UInt16 nItems;
} StringArrayType;

typedef struct {
	UInt8 nIngredients;
	IngredientNodePtr pIngredientList;
	UInt8 nDescriptors;
	UInt8 *pDescriptors;
	char *sName;
	char *sDirections;
} DrinkType;
typedef DrinkType* DrinkPtr;

typedef struct {
	UInt8 nDrinkDescriptors;
	char *pDrinkDescriptors;
	UInt8 nIngredientDescriptors;
	char *pIngredientDescriptors;
	UInt8 nIngredients;
	char *pIngredients;
} AppInfoBlockType;
typedef AppInfoBlockType* AppInfoBlockPtr;

//enum StatusEnum {
//	OFF,
//	ON
//};
//typedef enum StatusEnum FilterStatus;

enum TargetEnum {
	DRINK,
	INGREDIENT
};
typedef enum TargetEnum FilterTarget;

enum ActionEnum {
    OFF,
	INCLUDE,
	EXCLUDE
};
typedef enum ActionEnum FilterAction;

typedef struct {
	FilterTarget target;
	FilterAction action;
	UInt8 nDescriptors;
	UInt8 nIngredients;
	UInt16 nDescriptorSelection;
	UInt16 nIngredientSelection;
	Boolean bTypeChecked;
	char **psDescriptors;
	char **psIngredients;
} FilterType;
typedef FilterType* FilterPtr;

//===========================================================================
// Function Prototypes
//===========================================================================

Err AddDrinkRecord(UInt16 nIndex, DrinkPtr pDrink);
void AddIngredientCleanup(UInt8 nIngredients, UInt8 nDescriptors,
	char **psIngredients, char **psDescriptors, MemHandle hUnitsList,
	MemHandle hUnitsStrings);
void BuildIngredientString(AppInfoBlockPtr pInfo, IngredientType ingredient,
	char *sNew);
Int16 CompareDrinkRecords(UInt8 *pRec1, UInt8 *pRec2, Int16 nOther,
	SortRecordInfoPtr pRec1SortInfo, SortRecordInfoPtr pRec2SortInfo,
	MemHandle hAppInfo);
void CopyDrinkDescriptors(UInt8 *pBlock, UInt16 *pnOffset, AppInfoBlockPtr info);
void CopyIngredientDescriptors(UInt8 *pBlock, UInt16 *pnOffset, AppInfoBlockPtr info);
void CopyIngredients(UInt8 *pBlock, UInt16 *pnOffset, AppInfoBlockPtr info);
UInt32 DrinkPackedSize(DrinkPtr pDrink);
void DisplayInt(UInt32 n);
void DisplayString(char *s);
void DrawBoxedMessage(UInt16 x, UInt16 y, char *sMessage);
Err DrinkFree(DrinkPtr pDrink);
Boolean FilterDrink(FilterPtr *filter, DrinkPtr pDrink);
UInt8* GetAppInfoBlockPtr(void);
Err GetAppInfoBlock(AppInfoBlockPtr pInfo);
UInt32 GetAppInfoBlockSize(AppInfoBlockType *pInfo);
void GetDrinkList(DmOpenRef *pDB, FilterType **filter, char ***psDrinks,
	UInt16 **pnRecordIndices, UInt16 *nDrinks);
Err GetRecordHandle(DmOpenRef *pDB, UInt16 nRecord, MemHandle *h);
void GetStringModal(char* sTitle, char *sName);
void GetIngredients(AppInfoBlockPtr pInfo, IngredientNodePtr pIngredientList,
	char ***psIngredients, UInt8 nIngredients);
ControlType* GetControlPtr(FormPtr pForm, UInt16 nId);
UInt8 GetFractionalLabel(UInt8 nAmount);
Int16 RandInt(const int nMax);
UInt32 SetScreenMode(void);
void SetFieldText(FormPtr pForm, UInt16 nId, char *psz);
char** StrArrayInsert(char *sNew, UInt8 nPos, char *pSource, UInt8 *nItems);
void StrArrayMerge(char *sNew, char *pSource, char **psDest, UInt16 nItems);
void StrArrayFree(char **psArray, UInt16 nItems);
void StrArraySwap(char ***ppsArray, Int16 i, Int16 j);
void StrArraySort(char ***ppsArray, Int16 nLeft, Int16 nRight);
void StrPackedCopy(char *pSource, char **psDest, UInt16 nItems);
Int16 StrPackedFind(char *sInput, char *pSource, UInt16 nItems);
void UpdateFilterUI(FormPtr pForm, FilterType **filter, UInt8 nFilter);
void UpdateArrowState(FormPtr pForm, IngredientNodePtr pIngredientList,
	char *sUpLabel, char *sDownLabel);
void UpdateScrollbar(FieldPtr pField, ScrollBarPtr pScroll);
Err UnpackDrinkRecord(UInt8 *p, DrinkPtr pDrink);

// EOF 
